/*
 * macsdrawer.m --
 *
 *	This module implements custom drawer windows for Mac OS X.
 *
 * Copyright (c) 2010 Kevin Walzer/WordTech Communications LLC.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *  
 * 
 */

#import <tcl.h>
#import <tk.h>
#import <tkInt.h>
#import <tkMacOSXInt.h>
#import <Cocoa/Cocoa.h>


//declarations of functions

int MakeTkDrawer(ClientData clientData, Tcl_Interp *ip,
		 int objc, Tcl_Obj *CONST objv[]);
int DestroyTkDrawer (ClientData clientData, Tcl_Interp *ip,
		     int objc, Tcl_Obj *CONST objv[]);

int Macdrawer_Init (Tcl_Interp *ip);
int Macdrawer_SafeInit(Tcl_Interp *ip);


//run the drawer
int MakeTkDrawer(ClientData clientData, Tcl_Interp *ip,
		 int objc, Tcl_Obj *CONST objv[]) {

  if (objc != 3) {
    Tcl_WrongNumArgs(ip, 1, objv, "parent? drawer?");
    return TCL_ERROR;
  }


  //get handle on parent Cocoa window from Tk

  Tk_Window parentpath = Tk_NameToWindow(ip, Tcl_GetString(objv[1]), Tk_MainWindow(ip));

  if (parentpath == NULL) {
    return TCL_ERROR;
  }

  Drawable parent_d = Tk_WindowId(parentpath);
  NSView *parentview = TkMacOSXGetRootControl(parent_d);
  NSWindow *parent = [parentview window];

  //get handle on drawer Cocoa window from Tk

  Tk_Window drawerpath = Tk_NameToWindow(ip, Tcl_GetString(objv[2]), Tk_MainWindow(ip));

 if (drawerpath == NULL) {
    return TCL_ERROR;
  }

  Tk_MakeWindowExist(drawerpath);

  Drawable drawer_d = Tk_WindowId(drawerpath);
  NSView *drawerview = TkMacOSXGetRootControl(drawer_d);
  NSWindow *drawer = [drawerview window];


  //make the drawer window a child of the parent; this allows "grouping" so the window can be attached to the parent, move with parent
  [parent addChildWindow:drawer ordered:NSWindowBelow];

  return TCL_OK;


}

int DestroyTkDrawer (ClientData clientData, Tcl_Interp *ip,
		     int objc, Tcl_Obj *CONST objv[]) {


  if (objc != 3) {
    Tcl_WrongNumArgs(ip, 1, objv, "parent? drawer?");
    return TCL_ERROR;
  }

  //get handle on parent Cocoa window from Tk

  Tk_Window parentpath = Tk_NameToWindow(ip, Tcl_GetString(objv[1]), Tk_MainWindow(ip));

  if (parentpath == NULL) {
    return TCL_ERROR;
  }

  Drawable parent_d = Tk_WindowId(parentpath);
  NSView *parentview = TkMacOSXGetRootControl(parent_d);
  NSWindow *parent = [parentview window];


  Tk_Window drawerpath = Tk_NameToWindow(ip, Tcl_GetString(objv[2]), Tk_MainWindow(ip)); 
  Drawable drawer_d = Tk_WindowId(drawerpath);
  NSView *drawerview = TkMacOSXGetRootControl(drawer_d);
  NSDrawer *drawer = [drawerview window];

 if (drawerpath == NULL) {
    return TCL_ERROR;
  }

  
  //detatch the drawer from its parent
  [parent removeChildWindow: drawer];

  return TCL_OK;
}




//initalize the package in the tcl interpreter, create tcl commands
int Macdrawer_Init (Tcl_Interp *ip) {
	
  // set up an autorelease pool
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
  if (Tcl_InitStubs(ip, "8.5", 0) == NULL) {
    return TCL_ERROR;
  }


  if (Tk_InitStubs(ip, "8.5", 0) == NULL) {
    return TCL_ERROR;
  }

  Tcl_CreateObjCommand(ip, "::macdrawer::makedrawer", MakeTkDrawer,(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
  Tcl_CreateObjCommand(ip, "::macdrawer::removedrawer", DestroyTkDrawer,(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  if (Tcl_PkgProvide(ip, "macdrawer", "1.0") != TCL_OK) {
    return TCL_ERROR;
  }

  //release memory
  [pool release];
	
  return TCL_OK;
	

}

int Macdrawer_SafeInit(Tcl_Interp *ip) {
  return Macdrawer_Init(ip);
}



